# Overview

This module provides the foundation for integration with Orange Logic's [Orange DAM](https://www.orangelogic.com/products/digital-asset-management-system) to allow for synchronizing data to Drupal.

__NOTICE: This module is in pre-release and under active development. It should be considered a preview and not ready for production use.__

It provides the generic tooling and API integration required for any Orange DAM integration but also provides the flexibility to allow for the detailed field mapping that will be required for your specific needs.

The module monitors Orange DAM for changes, queues the changes it finds, prepares the updated data for migrations, allows the user to map fields in custom migrations, and manages the running of all of these processes via cron or manually via Drush commands.

# Setup

Follow the steps below to begin syncing data from your Orange DAM to Drupal:

## Debugging/Logging

There are several log channels that Orange Logic information is logged to:

- `orange_dam.migration` - Migration script activity and data issues.
- `orange_dam.api` - API request information and errors.
- `orange_dam.queue` - Queuing events and related problems.

There are many ways to view the logs. They can be viewed in the admin UI or via drush.
For example:
```bash
drush watchdog:show --type=orange_dam.migration
```

## API Credentials

The following configuration is used for configuring API requests. However, some of these items have default values provided in the configuration yml files.

Add the following to the `orange_dam.settings` configuration:

### Base Path

```yaml
base_path: https://example.orangelogic.com
```

### Query String

Optionally configure additional parameters as needed:

```yaml
query_string: 'UseSession=<YOUR ORANGE DAM SESSION ID>
```

### Client ID & Client Secret

The following two variables should likely not be tracked in the repo. They can either be set directly in a local settings file:

```php
$config['orange_dam.settings']['client_id'] = '<YOUR ORANGE DAM CLIENT ID>';
$config['orange_dam.settings']['client_secret'] = '<YOUR ORANGE DAM CLIENT SECRET>';
```

Or set using the following environment variables:

```
ORANGE_DAM_CLIENT_ID=<YOUR ORANGE DAM CLIENT ID>
ORANGE_DAM_CLIENT_SECRET=<YOUR ORANGE DAM SECRET>
```

Add the following to the `settings.php` file if environment variables are used:

```php
if ($orange_dam_client_id = getenv('ORANGE_DAM_CLIENT_ID')) {
  $config['orange_dam.settings']['client_id'] = $orange_dam_client_id;
}
if ($orange_dam_client_secret = getenv('ORANGE_DAM_CLIENT_SECRET')) {
  $config['orange_dam.settings']['client_secret'] = $orange_dam_client_secret;
}
```

## Field Configuration

The complete list of fields that your Orange DAM provides and that should be retrieved for import via the migrations should be provided in configuration.

Add the following to the `orange_dam.settings` configuration:

### Base Fields

The `base_fields` value represents the essential fields that are needed to represent items that need to be updated.

The likely values are:

```yaml
base_fields:
  - Title
  - SystemIdentifier
  - CoreField.DocSubType
```

### All Fields

The `all_fields` value represents the complete list of fields that should be retrieved for all content types when making API requests.

```yaml
all_fields:
  - Title
  - SystemIdentifier
  - CoreField.DocSubType
  - ExampleField
  - AnotherCustomField
```

## Building Migrations

A migration should be created for each item type represented in your Orange DAM. This module uses the [migrate_source_queue module](https://www.drupal.org/project/migrate_source_queue) and requires the creation of queue-sourced migrations for your content.

The queues used as the source for these migrations are [registered dynamically](./src/Plugin/Derivative/OrangeDamMigrationQueues.php) by searching for all migrations tagged with `orange_dam`.

### Migration Example

When creating the migrations the following should be considered:

- The migration `id` should be based on an snake-cased form of the `CoreField.DocSubType` property of the Orange Logic API response and prefixed with `orange_dam_`. All non-word characters, including dashes, will be replaced with an underscore.
  - Examples:
    - if the `CoreField.DocSubType` is "Audio Container", the migration ID will be `orange_dam_audio_container`.
    - if the `CoreField.DocSubType` is "Document+Nav/Sub Folder", the migration ID will be `orange_dam_document_nav_sub_folder`.
  - The migration configuration file name must match the migration `id` as well. So, if your id is `orange_dam_audio_container`, your migration file should be `migrate_plus.migration.orange_dam_audio_container.yml`.
  - In addition, the queue name configured in the migration file must also match the id. So, following our example, the `queue_name` property for an 'Audio Container' DocSubType would be `queue_name: 'orange_dam_migration:orange_dam_audio_container'`.
- Each migration should be tagged with `orange_dam` within its `tags` configuration.
- Content migrations should be tagged with `orange_dam_content` and taxonomy based migrations should be tagged with `orange_dam_taxonomy`. This will not affect their functionality, but it will aid in filtering reporting data in drush command output.

`migrate_plus.migration.<example_item_name>.yml`:

```yaml
uuid: <UUID>
...
id: <example_item_name>
...
migration_tags:
  - orange_dam
migration_group: orange_dam
...
source:
  plugin: queue
  queue_name: 'orange_dam_migration:<example_item_name>'
...
```

### Migration Tools

Various `@MigrateProcessPlugin` [plugins](./src/Plugin/migrate/process) are provided by this module to aid in the development of migrations.

## Cron Integration

This modules does not provide a cron integration, as the timing of the calls is likely something that is highly unique for each site. If you are using `advanced_cron`, the following configuration is suggested for a sync every 5 minutes.
```yaml
-langcode: en
-status: true
-dependencies:
-  module:
-    - orange_dam
-title: 'Orange DAM - Sync'
-id: orange_dam_sync
-weight: 0
-module: orange_dam
-callback: 'orange_dam.queue_data_manager:syncUpdates'
-scheduler:
-  id: simple
-  configuration:
-    rules:
-      - '*/5+@ * * * *'
-launcher:
-  id: serial
-  configuration:
-    timeouts:
-      lock_timeout: 3600
-    launcher:
-      thread: 0
-logger:
-  id: database
-  configuration:
-    method: '3'
-    expire: 1209600
-    retain: 1000
```

# Sync Process

1. Items that need to be created or updated are retrieved and then sorted (based upon their `CoreField.DocSubType`value) into separate migration queues. Checking for updated items can be done automatically via cron or it can be run manually via Drush commands (documented below) that retrieve Orange DAM items based upon a variety of configurable parameters.
1. You can examine the state of the queues with the `drush orange-dam:sync-status` command. You will see a table listing all the destination queues:

```shell
$ drush orange-dam:sync-status
...
------------------------------------ ------------
  Queue Name                           Item Count
 ------------------------------------ ------------
 Orange DAM - foo queue                    5
 Orange DAM - bar queue                    197
 ...
```

1. Run the migrations to create/update the content. To run the migrations use the command provided by this module or the standard migrate commands that come with the [migrate_tools module](https://www.drupal.org/project/migrate_tools).

   _Using the command provided by this module ensures that the proper order of operations is followed._

```shell
$ drush orange-dam:run-migration-queues
```

or, using standard migration commands.

```shell
$ drush migrate:import <name/id of your Orange DAM queue migration>
```

# Drush Commands

To see all Orange DAM commands run the following:
```shell
drush list --filter=orange-dam
```

Each command provides additional documentation about how it should be used.

```shell
drush orange-dam:example-command -h
```

# Road Map - Future Plans

- An interface that allows for detailed visibility into the state of the various operations.
- A way for important items to cut in line and be fully processed in a single command.
- Delete functionality integration based upon an objects visibility class.
- Authority list tag syncing for taxonomy based fields.
